#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import args_to_dtype

from .rolling import rolling

import pandas as pd


@args_to_dtype(pd.DataFrame)
def cmf(df, n=2):
    ''' Returns the Chaikin Money Flow of a OHLCV dataframe'''

    high = df['high'].values
    low = df['low'].values
    close = df['close'].values
    volume = df['volume'].values

    avg = (2*close - high - low) / (high - low + .000000001) * volume

    avg_roll = rolling(avg, fn='sum', n=n)
    vol_roll = rolling(volume, fn='sum', n=n)

    _chaikin_money_flow = avg_roll / vol_roll
    _chaikin_money_flow[:n] = 0

    return _chaikin_money_flow



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

import itertools
import numpy as np


def fill_values(averages, interval):
    ''' Fill missing values with evenly spaced samples.

    Example: You're using 15-min candlestick data but want to include a 1-hour moving
        average with a value at every 15-min mark, and not just every 1-hour mark.
    '''

    # Combine every two values with the number of intervals between each value
    avgs_zip = zip(averages[:-1], averages[1:], itertools.repeat(interval), itertools.repeat(False))

    # Generate evenly-spaced samples between every point
    avgs_gen = (np.linspace(*x, end) for x in avgs_zip)

    # Unpack all values into one list
    _fill_values = list(itertools.chain.from_iterable(avgs_gen))

    return _fill_values



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import args_to_dtype

import numpy as np


@args_to_dtype(list)
def rsi(src, n=2):
    ''' Returns the "relative strength index", which is used to measure the velocity
    and magnitude of directional price movement. '''

    deltas = np.diff(src)
    seed = deltas[:n+1]
    up = seed[seed > 0].sum()/n
    down = -seed[seed < 0].sum()/n
    _rsi = np.zeros_like(src)
    _rsi[:n] = 100. - 100./(1.+ up/down)
    for i in range(n, len(src)):
        delta = deltas[i-1]
        if delta > 0:
            up_val = delta
            down_val = 0
        else:
            up_val = 0
            down_val = -delta

        up = (up*(n-1) + up_val)/n
        down = (down*(n-1) + down_val)/n

        _rsi[i] = 100. - 100./(1. + up/down)

    return _rsi



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import *

from .atr import *
from .bollinger import *
from .cmf import *
from .double_smooth import *
from .ema import *
from .ichimoku import *
from .macd import *
from .mmo import *
from .renko import *
from .roc import *
from .rolling import *
from .rsi import *
from .sdev import *
from .sma import *
from .td_sequential import *
from .tsi import *

import os
import requests
from datetime import datetime, timedelta
import pandas as pd
import numpy as np

import matplotlib.dates as mdates
pd.plotting.register_matplotlib_converters()



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from . import indicators as indicators
from . import plot as plot
from . import utils as utils

from .wrappers import *

import numpy as np
import pandas as pd

# Ignore stupid divide by 0 warnings
np.seterr(divide='ignore', invalid='ignore')

import matplotlib.dates as mdates
pd.plotting.register_matplotlib_converters()



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from .area_between import area_between
from .crossover import crossover
from .draw_candlesticks import draw_candlesticks
from .fill_values import fill_values
from .group_candles import group_candles
from .intersection import intersection
from TAcharts.wrappers import *



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import args_to_dtype

import pandas as pd


@args_to_dtype(pd.DataFrame)
def group_candles(df, interval=4):
    ''' Combine candles so instead of needing one dataset for each time interval,
        you can form time intervals using more precise data.

    Example: You have 15-min candlestick data but want to test a strategy based
        on 1-hour candlestick data  (interval=4).
    '''

    columns = ['date', 'open', 'high', 'low', 'close', 'volume']

    try:
        candles = df[columns].values
        results = []
        for i in range(0, len(df)-interval, interval):
            results.append([
                candles[i, 0],                      # date
                candles[i, 1],                      # open
                candles[i:i+interval, 2].max(),     # high
                candles[i:i+interval, 3].min(),     # low
                candles[i+interval, 4],             # close
                candles[i:i+interval, 5].sum()      # volume
            ])
        return pd.DataFrame(results, columns=columns)

    # File does not contain all columns needed to group candles
    except KeyError as e:
        raise KeyError(f'Column headers not compatable.  You need to have at least \
                        a "date", "open", "high", "low", "close", and "volume" column')



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.utils.draw_candlesticks import draw_candlesticks
from TAcharts.utils.demo_df import demo_df


def plot(self):
    fig, ax = plt.subplots(1, figsize=(20, 10))

    if date:
        x = self.date
    else:
        x = range(len(self.close))
    plt.plot(x, self.bollinger['sma'], label='Simple Moving Average', color='orange')
    plt.plot(x, self.bollinger['l_band'], color='blue', label=None)
    plt.plot(x, self.bollinger['h_band'], color='blue')

    draw_candlesticks(ax, self.df)

    plt.rc('axes', labelsize=20)
    plt.rc('font', size=18)

    ax.xaxis.set_major_formatter(mdates.DateFormatter('%m/%d/%y'))
    ax.xaxis.set_major_locator(mdates.MonthLocator())
    plt.xticks(rotation=45)

    fig.suptitle('Bollinger Bands', fontsize=30)
    plt.ylabel('BTC price ($)')
    plt.legend(f'{self.n}MA')

    return plt.show()



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import pd_series_to_np_array

import numpy as np


@pd_series_to_np_array
def roc(src, n=2):
    ''' Returns the rate of change in price over n periods '''

    _roc = np.zeros_like(src)
    _roc[n:] = (np.diff(src, n)) / src[:-n]

    return _roc



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import args_to_dtype

import pandas as pd


@args_to_dtype(pd.Series)
def sdev(src, n=2):
    'Returns the standard deviation of a `src` given `n` periods'

    _sdev = src.rolling(n).std().values
    return _sdev



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.utils.draw_candlesticks import draw_candlesticks
from TAcharts.utils.demo_df import demo_df

import matplotlib.pyplot as plt
import pandas as pd


def plot(df, ichimkou):
    fig, ax = plt.subplots(figsize=(30,10))

    x = df['date']
    try:
        x = pd.to_datetime(x)
    except:
        x = range(len(df))

    plt.plot(x, ichimoku['tenkan'], color='blue')
    plt.plot(x, ichimoku['kijun'], color='maroon')
    plt.plot(x, ichimoku['senkou_a'], color='green', linewidth=0.5)
    plt.plot(x, ichimoku['senkou_b'], color='red', linewidth=0.5)

    draw_candlesticks(ax, df)

    fig.suptitle('Ichimoku', fontsize=30)
    plt.ylabel('BTC price ($)')
    plt.rc('axes', labelsize=20)
    plt.rc('font', size=18)

    ax.xaxis.set_major_locator(mdates.MonthLocator())
    ax.xaxis.set_major_formatter(mdates.DateFormatter('%m/%d/%y'))
    plt.xticks(rotation=45)

    plt.fill_between(
        x, ichimoku['senkou_a'], ichimoku['senkou_b'],
        where=ichimoku['senkou_a'] >= ichimoku['senkou_b'],
        facecolor='limegreen',
        interpolate=True
    )
    plt.fill_between(
        x, ichimoku['senkou_a'], ichimoku['senkou_b'],
        where=ichimoku['senkou_a'] <= ichimoku['senkou_b'],
        facecolor='salmon',
        interpolate=True
    )
    return plt.show()



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import pd_series_to_np_array

import pandas as pd
import numpy as np

import numpy.lib.stride_tricks as stride_tricks

@pd_series_to_np_array
def rolling(src, n=2, fn=None, axis=1):
    ''' Returns the rolling sum, max, or min for a list across n periods '''

    if len(src) == 0:
        raise ValueError('`src` is length 0')
    # rolling sum
    if fn == 'sum':
        _rolling = src.cumsum()
        _rolling[n:] = _rolling[n:] - _rolling[:-n]
        _rolling[:n] = 0
        return _rolling
    # rolling max, min, or mean
    elif fn in ['max', 'min', 'mean']:
        shape = (len(src) - n + 1, n)
        strides = src.strides * 2
        src_strided = stride_tricks.as_strided(src, shape=shape, strides=strides)
        _rolling = np.zeros_like(src)
        _rolling[n-1:] = getattr(np, fn)(src_strided, axis=axis)
        return _rolling
    else:
        raise ValueError('Enter "sum", "max", "min", or "mean" as fn argument')



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import args_to_dtype

from .roc import roc
from .double_smooth import double_smooth

import pandas as pd


@args_to_dtype(pd.Series)
def tsi(src, slow=25, fast=13):
    ''' Returns the "true strength indicator", which is used to determine overbought
    and oversold conditions, and warning of trend weakness through divergence. '''

    _roc = roc(src, n=1)

    roc_double_smooth = double_smooth(_roc, slow=slow, fast=fast)
    roc_double_smooth_abs = double_smooth(abs(_roc), slow=slow, fast=fast)

    _tsi = (roc_double_smooth / roc_double_smooth_abs) * 100

    return _tsi



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import pd_series_to_np_array

from .rolling import rolling


@pd_series_to_np_array
def sma(src, n=2):
    ''' Returns the "simple moving average" for a list across n periods'''

    summed = rolling(src, fn='sum', n=n)
    _sma = summed / n

    return _sma



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.utils.group_candles import group_candles

from datetime import datetime, timedelta
import matplotlib.dates as mdates


import pandas as pd


class ichimoku:

    def __init__(self, df=None, filename=None, interval=None):

        if filename:
            df = pd.read_csv(os.getcwd() + '/' + filename)

        # Now that we have our df, make all headers lowercase
        df.columns = [col.lower() for col in df]

        if interval:
            df = group_candles(df, interval)

        self.df = df
        self.ichimoku = {}

        self._validate_data(interval)


    def _validate_data(self, interval):
        ''' Make sure we have enough data to form the cloud '''

        if interval > len(self.df):
            raise AssertionError(f'Error: make sure the dataset has more than {interval} rows.')


    def _extend_window(self, displacement):
        last_date = self.df['date'].iat[-1]
        window = last_date - self.df['date'].iat[-2]
        new_dates = [last_date + window*(1+i) for i in range(displacement)]
        self.df = self.df.append(pd.DataFrame(new_dates, columns=['date']), ignore_index=True, sort=False)
        return


    def _build_lines(self, **kwargs):
        for key, val in kwargs.items():
            high = sma(self.df['high'], val, 'max')
            low = sma(self.df['low'], val, 'min')
            self.ichimoku[key] = (high + low)/2
        return


    def _build_spans(self, senkou_b_interval, displacement):
        self.ichimoku['chikou'] = self.df['close'].shift(-displacement)
        self.ichimoku['senkou_a'] = (self.ichimoku['tenkan'] + self.ichimoku['kijun']) / 2

        self._build_lines(senkou_b=senkou_b_interval)

        self.ichimoku['senkou_a'] = self.ichimoku['senkou_a'].shift(displacement)
        self.ichimoku['senkou_b'] = self.ichimoku['senkou_b'].shift(displacement)

        for indicator in ['tenkan', 'kijun', 'chikou', 'senkou_a', 'senkou_b']:
            self.ichimoku[indicator] = self.ichimoku[indicator][senkou_b_interval+displacement:]

        self.df = self.df[senkou_b_interval+displacement:]
        return


    def build(self, tenkan_interval, kijun_interval, senkou_b_interval, displacement):
        ''' Create Ichimoku data '''

        self._validate_data(senkou_b_interval)
        self._extend_window(displacement)

        # Tenkan (conversion line) = (highest high + highest low)/2 for the past x intervals
        # Kijun (base line) = (highest high + lowest low)/2 for the past x*3 intervals
        self._build_lines(tenkan=tenkan_interval, kijun=kijun_interval)

        # Chikou (lagging span) = Current closing price time-shifted backwards x*3 intervals
        # Senkou span A (leading span A) = (tenkan + kijun)/2 time-shifted forwards x*3 intervals
        # Senkou span B (leading span B) = (highest high + lowest low)/2 for past x*6 intervals, shifted forwards x*3 intervals
        self._build_spans(senkou_b_interval=senkou_b_interval, displacement=displacement)

        return self.ichimoku



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

import pandas as pd
import numpy as np


def args_to_dtype(dtype):
    ''' Convert arguments in a function to a specific data type, depending on what
        actions will be done with the arguments '''

    def format_args(fn):
        def wrapper(*args, **kwargs):
            args = [dtype(x) if type(x) != dtype else x for x in args]
            return fn(*args, **kwargs)
        return wrapper
    return format_args


def pd_series_to_np_array(fn):
    ''' Convert pandas.Series objects to numpy.array objects.  pd.Series.values is
    10x quicker than np.array(pd.Series) '''

    def wrapper(*args, **kwargs):
        args = [pd.Series(x).values if type(x) != pd.Series else x.values for x in args]
        return fn(*args, **kwargs)
    return wrapper



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import pd_series_to_np_array

import numpy as np


@pd_series_to_np_array
def crossover(x1, x2):
    ''' Find all instances of intersections between two lines '''

    x1_gt_x2 = x1 > x2
    cross = np.diff(x1_gt_x2)
    cross = np.insert(cross, 0, False)
    cross_indices = np.flatnonzero(cross)
    return cross_indices



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import pd_series_to_np_array

import numpy as np


@pd_series_to_np_array
def area_between(line1, line2):
    ''' Return the area between line1 and line2 '''

    diff = line1 - line2
    x1 = diff[:-1]
    x2 = diff[1:]

    triangle_area = sum(abs(x2 - x1) * .5)
    square_area = sum(np.min(zip(x1, x2), axis=1))
    _area_between = triangle_area + square_area

    return _area_between



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

import matplotlib.pyplot as plt
from matplotlib import colors as mcolors
from matplotlib.lines import Line2D
from matplotlib.patches import Rectangle
from matplotlib.dates import date2num


def draw_candlesticks(ax, df):
    ''' Add candlestick visuals to a matplotlib graph '''

    df = df[['date', 'open', 'high', 'low', 'close']].dropna()
    lines = []
    patches = []

    for i, (date, _open, high, low, close) in df.iterrows():
        date = date2num(date)
        if close >= _open:
            color = 'g'
            lower = _open
            height = close - _open
        else:
            color = 'r'
            lower = close
            height = _open - close

        vline = Line2D(
            xdata=(date, date), ydata=(low, high),
            color=color,
            linewidth=0.5,
            antialiased=True
        )

        rect = Rectangle(
            xy=(date - .4, lower),
            width=0.8,
            height=height,
            facecolor=color,
            edgecolor=color,
            alpha=1.0
        )

        lines.append(vline)
        ax.add_line(vline)
        patches.append(rect)
        ax.add_patch(rect)

    ax.autoscale_view()
    return lines, patches



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import pd_series_to_np_array

import numpy as np


@pd_series_to_np_array
def td_sequential(src, n=2):
    ''' Returns the TD sequential of the close '''

    old_gt_new = src[:-n] > src[n:]
    diff_lst = np.diff(old_gt_new)
    diff_lst = np.insert(diff_lst, 0, False)

    _td_sequential = [0 for _ in range(n)]

    for diff in diff_lst:
        if not diff:
            _td_sequential.append(_td_sequential[-1] + 1)
        else:
            _td_sequential.append(1)

    return _td_sequential



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.utils.demo_df import demo_df

import matplotlib.pyplot as plt
import matplotlib.patches as patches

import pandas as pd



def plot(renko, num_bricks=None, signal_indices=None):

    prices = renko['price']
    directions = renko['direction']
    brick_size = renko['brick_size']

    # Limits display to a custom sized set
    if num_bricks:
        prices = prices[-num_bricks:]
        directions = directions[-num_bricks:]

    if signal_indices:
        for index in signal_indices:
            plt.axvline(x=index)

    # Create `fig` and `ax`
    fig, ax = plt.subplots(1, figsize=(10, 5))

    # Add title to chart
    fig.suptitle(f"Renko Chart (brick size = {round(brick_size, 2)})", fontsize=20)

    # Add label to Y-axis
    ax.set_ylabel('Price ($)')

    plt.rc('axes', labelsize=20)
    plt.rc('font', size=16)

    # Setup X-axis
    x_min = 0
    x_max = len(prices) + 1
    ax.set_xlim(x_min, x_max)

    # Setup Y-axis
    y_min = min(prices) - 2 * brick_size
    y_max = max(prices) + 2 * bricksize
    ax.set_ylim(y_min, y_max)

    # Add bricks to chart
    for x, (price, direction) in enumerate(zip(prices, directions)):

        # Setting brick color and y-position
        if direction == 1:
            facecolor='g'
            y = price - brick_size
        else:
            facecolor = 'r'
            y = price

        ax.add_patch(
            patches.Rectangle(
                (x+1, y),
                height=brick_size,
                width=1,
                facecolor=facecolor # Either Green or Red
            )  # end of patches.Rectangle
        )   # end of ax.add_patch

    return plt.show()



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

import TAcharts

import os
import io
import requests
import pandas as pd


class OHLCV:

    path = f'{TAcharts.__path__[0]}/data'
    url = 'https://raw.githubusercontent.com/carlfarterson/TAcharts/master/data/btc-2019.csv'

    def __init__(self, url=None, usecols=None):
        self.url = url
        self.usecols = usecols
        self._add_2019_hourly_ohlcv()

    def _add_2019_hourly_ohlcv(self):
        for csvfile in os.listdir(self.path):
            coin = csvfile[:csvfile.find('.')]
            df = pd.read_csv(f'{self.path}/{csvfile}', usecols=self.usecols)
            setattr(self, coin, df)


    def fetch_ohlcv(self, url=None, usecols=None):

        # No url provided means we use our btc price file
        # Fetch prices with URL provided
        content = requests.get(url).content
        ohlcv = pd.read_csv(io.StringIO(content.decode('utf-8')), usecols=usecols)



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import args_to_dtype

import pandas as pd


@args_to_dtype(pd.Series)
def ema(src, n=2):
    ''' Returns the "exponential moving average" for a list '''

    _ema = src.ewm(span=n, min_periods=1, adjust=False).mean()

    return _ema.values



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import pd_series_to_np_array

from .rolling import rolling


@pd_series_to_np_array
def mmo(src, n=2):
    ''' Returns the Murrey Math Oscillator of the close '''

    # Donchian channel
    highest = rolling(src, fn='max', n=n)
    lowest = rolling(src, fn='min', n=n)

    rng = highest - lowest

    # Oscillator
    rng_multiplier = rng * .125
    midline = lowest + rng_multiplier * 4

    _mmo = (src - midline) / rng
    _mmo[:n] = 0

    return _mmo



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import pd_series_to_np_array

from .rolling import rolling
from .sma import sma

import numpy as np
import pandas as pd


def atr(high, low, close, n=14):
    ''' Returns the average true range from candlestick data '''

    prev_close = np.insert(close[:-1], 0, 0)
    # True range is largest of the following:
    #   a. Current high - current low
    #   b. Absolute value of current high - previous close
    #   c. Absolute value of current low - previous close
    true_range = np.max([high-low, abs(high-prev_close), abs(low-prev_close)])
    true_range = rolling(high - low, abs(high - prev_close), abs(low - prev_close), fn='max', n=1, axis=0)

    _atr = sma(true_range, n)

    return _atr



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-

import os.path
import codecs
import setuptools

# Get the long description from the relevant file
here = os.path.abspath(os.path.dirname(__file__))
with codecs.open(os.path.join(here, 'README.md'), encoding='utf-8') as readme:
    long_description = readme.read()

setuptools.setup(
  name = 'TAcharts',
  version = '0.0.8',
  author = 'Carter Carlson',
  author_email = 'carlfarterson@gmail.com',
  license = 'MIT',
  description = 'TA Charting tool',
  keywords=['TA', 'mathematics', 'algorithms'],
  long_description=long_description,
  long_description_content_type='text/markdown',
  url = 'https://github.com/carlfarterson/TAcharts',
  packages=setuptools.find_packages(),
  download_url = 'https://github.com/carlfarterson/TAcharts/archive/v_0.0.1.tar.gz',
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Programming Language :: Python :: 3',
    'License :: OSI Approved :: MIT License',
    'Intended Audience :: Developers'
  ],
  python_requires='>=3.6'
)



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import args_to_dtype
from .ema import ema

import pandas as pd


@args_to_dtype(pd.Series)
def macd(src, slow=25, fast=13):
    ''' Returns the "moving average convergence/divergence" (MACD) '''

    ema_fast = ema(src, fast)
    ema_slow = ema(src, slow)
    _macd = ema_fast.values - ema_slow.values

    return _macd



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals


def intersection(a0, a1, b0, b1):
    ''' Return the intersection coordinates between vector A and vector B '''

    a_diff = a1 - a0
    b_diff = b1 - b0

    pos0_diff = float(a0 - b0)

    x = pos0_diff / (b_diff - a_diff)
    y = b_diff*x + b0

    return x, y



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.utils.group_candles import group_candles

from .sma import sma
from .sdev import sdev

import pandas as pd
import os


def bollinger(df=None, filename=None, interval=None, n=20, ndev=2):

    if filename:
        filename_abs_path = f'{os.getcwd()}/{filename}'
        try:
            df = pd.read_csv(filename_abs_path)
        except:
            raise FileNotFoundError(f'{filename_abs_path}\n\nDoes not exist.')
    elif df is None:
        df = demo_df

    # Make all columns lowercase
    df.columns = [col.lower() for col in df]

    if interval:
        df = group_candles(df, interval)

    # Make sure we have more rows of data than the size of the rolling average
    if len(df) < n:
        raise AssertionError('Rolling average cannot be larger than number of records.')

    df['date'] = df['date'].shift(1)

    _bollinger = {}

    # Create base simple moving average
    _bollinger['sma'] = sma(df['close'], n=n)

    rng = ndev * sdev(df['close'], n=n)

    # Create upper and lower bands
    _bollinger['h_band'] = _bollinger['sma'] + rng
    _bollinger['l_band'] = _bollinger['sma'] - rng

    return _bollinger



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import args_to_dtype
from .ema import ema

import pandas as pd


@args_to_dtype(pd.Series)
def double_smooth(src, slow=25, fast=13):
    ''' Returns the smoothed value of two EMAs '''

    first_smooth = ema(src, n=slow)
    _double_smooth = ema(first_smooth, n=fast)

    return _double_smooth.values



# Practicing with *args
import pandas as pd
import timeit

from utils.area_between import area_between
from utils.crossover import crossover
from indicators.sma import sma


import itertools
import numpy as np



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.utils.demo_df import demo_df

from .atr import atr

import numpy as np
import pandas as pd
import os


class renko:
    def __init__(self, df=None, filename=None, interval=None):
        if filename:
            filename_abs_path = f'{os.getcwd()}/{filename}'
            try:
                df = pd.read_csv(filename_abs_path)
            except:
                raise FileNotFoundError(f'{filename_abs_path}\n\nDoes not exist.')
        elif df is None:
            df = demo_df

        if interval:
            df = group_candles(df, interval)

        df['date'] = df['date'].shift(1)
        self.df = df


    def set_brick_size(self, brick_size=None, auto=True, atr_interval=14):
        ''' Setting brick size '''

        if len(self.df) < atr_interval:
            raise ValueError('ATR interval is longer than historical data.')

        self.brick_size = self._optimize_brick_size(auto, brick_size, atr_interval)
        return self.brick_size


    def _optimize_brick_size(self, auto, brick_size, atr_interval):
        ''' Helper function to get optimal brick size based on ATR '''

        if auto and not brick_size:
            average_true_range = atr(self.df['high'], self.df['low'], self.df['close'], n=atr_interval)
            brick_size = np.median(average_true_range)
        return brick_size


    def _apply_renko(self, i):
        ''' Determine if there are any new bricks to paint with current price '''

        num_bricks = 0

        gap = (self.close[i] - self.renko['price'][-1]) // self.brick_size
        direction = np.sign(gap)
        # No gap means there's not a new brick
        if direction == 0:
            return
        # Add brick(s) in the same direction
        if (gap > 0 and self.renko['direction'][-1] >= 0) \
        or (gap < 0 and self.renko['direction'][-1] <= 0):
            num_bricks = gap
        # Gap >= 2 or -2 and opposite renko direction means we're switching brick direction
        elif np.abs(gap) >= 2:
            num_bricks = gap - 2*direction
            self._update_renko(i, direction, 2)

        for brick in range(abs(int(num_bricks))):
            self._update_renko(i, direction)

        return


    def _update_renko(self, i, direction, brick_multiplier=1):
        ''' Append price and new block to renko dict '''

        renko_price = self.renko['price'][-1] + (direction * brick_multiplier * self.brick_size)
        self.renko['index'].append(i)
        self.renko['price'].append(renko_price)
        self.renko['direction'].append(direction)
        self.renko['date'].append(self.df['date'].iat[i])
        return


    def build(self):
        ''' Create Renko data '''

        units = self.df['close'].iat[0] // self.brick_size
        start_price = units * self.brick_size

        # Create Genesis block
        self.renko = {
            'index': [0],
            'date': [self.df['date'].iat[0]],
            'price': [start_price],
            'direction': [0]
        }

        # Run price feed through our renko model
        for i in range(1, len(self.close)):
            self._apply_renko(i)

        return self.renko
